package com.yizhi.calendar.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.calendar.application.vo.CalendarRecordVO;
import com.yizhi.calendar.application.vo.CalendarSignViewVO;
import com.yizhi.calendar.application.vo.HeadVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

@FeignClient(name = "calendar", contextId = "StudentCalendarClient")
public interface StudentCalendarClient {
	
	/**
	 * 打卡签到入口
	 * @param accountId
	 * @param siteId
	 * @return
	 */
	@GetMapping("/api/calendar/sign")
	public Map<String, Object> sign(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId);
	
	/**
	 * 用户每日打卡详情
	 * @param accountId
	 * @param siteId
	 * @return
	 */
	@GetMapping("/api/calendar/isSign")
	public CalendarSignViewVO isSign(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId);
	
	/**
	 * 签到打卡头部数据
	 * @param accountId
	 * @param siteId
	 * @return
	 */
	@GetMapping("/api/calendar/headData")
	public HeadVO getHeadData(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId);

	/**
	 * 签到明细
	 * @param accountId
	 * @param siteId
	 * @param type
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/api/calendar/signDetail")
	public Page<CalendarRecordVO> getSignDetail(@RequestParam("accountId") Long accountId, @RequestParam("siteId") Long siteId, @RequestParam("type") Integer type, @RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize);

	/**
	 * 签到日历
	 * @param accountId
	 * @param siteId
	 * @param month
	 * @return
	 */
	@GetMapping("/api/calendar/signByMonth")
	public List<CalendarRecordVO> getSignByMonth(@RequestParam("accountId") Long accountId, @RequestParam("siteId") Long siteId, @RequestParam("month") String month);

	/**
	 * 周期内用户打卡次数
	 * @param accountId
	 * @param siteIds
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	@GetMapping("/api/calendar/signNum")
	public Map<Long, Long>  signNum(@RequestParam("siteIds") List<Long> siteIds,@RequestParam("startDate") String startDate,@RequestParam("endDate") String endDate);

	@GetMapping("/api/calendar/getSignState")
	public Integer getSignState(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId);
}
