package com.yizhi.calendar.application.controller;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.calendar.application.vo.CalendarPointSetVO;
import com.yizhi.calendar.application.domain.Calendar;
import com.yizhi.calendar.application.domain.CalendarPointSet;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.calendar.application.service.ICalendarPointSetService;
import com.yizhi.calendar.application.service.ICalendarService;
import com.yizhi.calendar.application.vo.CalendarParam;
import com.yizhi.calendar.application.vo.CalendarPointSetParam;
import com.yizhi.calendar.application.vo.CalendarVO;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * <p>
 * 日历打卡积分发放设置表 前端控制器
 * </p>
 *
 * @author fulan123
 * @since 2019-10-10
 */
@RestController
@RequestMapping("/manage/calendar")
public class CalendarController {

	@Autowired
	private ICalendarService calendarService;
	
	@Autowired
	private ICalendarPointSetService calendarPointSetService;
	
	@Autowired
	private IdGenerator idGenerator;
	
	private static final Logger LOG = LoggerFactory.getLogger(CalendarController.class);
	
	/**
	 * 日历签到打卡规则回显
	 * @return
	 */
	@GetMapping("/calendarPointSet/getCalendar")
	public CalendarVO getCalendar() {
		CalendarVO vo = new CalendarVO();
		RequestContext res = ContextHolder.get();
		Long siteId = res.getSiteId();
		Calendar set = new Calendar();
		set.setSiteId(siteId);
		EntityWrapper<Calendar> wrapper = new EntityWrapper<Calendar>(set);
		Calendar calendar = calendarService.selectOne(wrapper);
		if (null!=calendar) {
			CalendarPointSet pointSet = new CalendarPointSet();
			pointSet.setCalendarId(calendar.getId());
			pointSet.setSetType(calendar.getSetType());
			pointSet.setState(1);
			EntityWrapper<CalendarPointSet> entityWrapper = new EntityWrapper(pointSet);
			entityWrapper.orderBy("minDate", true);
			List<CalendarPointSet> list = calendarPointSetService.selectList(entityWrapper);

			List<CalendarPointSetVO> calendarPointSetVOS = new ArrayList<CalendarPointSetVO>();
			for (CalendarPointSet calendarPointSet : list) {
				CalendarPointSetVO calendarPointSetVO = new CalendarPointSetVO();
				BeanUtils.copyProperties(calendarPointSet,calendarPointSetVO);
				calendarPointSetVOS.add(calendarPointSetVO);
			}
			vo.setSets(calendarPointSetVOS);
			vo.setId(calendar.getId());
			vo.setFirstPoint(calendar.getFirstPoint());
			vo.setSetType(calendar.getSetType());
			vo.setCycle(calendar.getCycle());
			vo.setState(calendar.getState());
		}else {
			CalendarPointSet pointSet = new CalendarPointSet();
			pointSet.setSetType(1);
			pointSet.setPoint(1);
			List<CalendarPointSetVO> list = new ArrayList<CalendarPointSetVO>();
			CalendarPointSetVO pointSetVO = new CalendarPointSetVO();
			BeanUtils.copyProperties(pointSet,pointSetVO);
			list.add(pointSetVO);
			vo.setFirstPoint(1);
			vo.setSetType(1);
			vo.setCycle(7l);
			vo.setSets(list);
			vo.setState(0);
		}
		return vo;
	}
	
	@PostMapping(value = "/calendarPointSet/insert")
	public Boolean insertCalendarPointSet(@RequestBody CalendarParam calendarParam){
		LOG.info("入参："+calendarParam.toString());
        String result = "";
		RequestContext res = ContextHolder.get();
		Long siteId = res.getSiteId();
		Long accountId = res.getAccountId();
		String accountName = res.getAccountName();
		Long companyId = res.getCompanyId();
		Long orgId = res.getOrgId();
		Calendar set = new Calendar();
		set.setSiteId(siteId);
		EntityWrapper<Calendar> wrapper = new EntityWrapper<Calendar>(set);
		Calendar calendar = calendarService.selectOne(wrapper);
		//第一次设置日历打卡规则
		if (null==calendar) {
			calendar = new Calendar();
			Long id = idGenerator.generate();
			calendar.setId(id);
			calendar.setSetType(calendarParam.getSetType());
			calendar.setFirstPoint(calendarParam.getFirstPoint());
			calendar.setCycle(calendarParam.getCycle());
			calendar.setState(calendarParam.getState());
			calendar.setCompanyId(companyId);
			calendar.setSiteId(siteId);
			calendar.setOrgId(orgId);
			calendar.setCreateById(accountId);
			calendar.setCreateByName(accountName);
			calendar.setCreateTime(new Date());
			calendarService.insert(calendar);
			for(CalendarPointSetParam pointSet:calendarParam.getCalendarPointSets()) {
				CalendarPointSet calendarPointSet = new CalendarPointSet();
				calendarPointSet.setId(idGenerator.generate());
				calendarPointSet.setCalendarId(id);
				calendarPointSet.setSetType(calendarParam.getSetType());
				if (calendarParam.getSetType()==1) {
					calendarPointSet.setPoint(pointSet.getPoint());
				}else {
					if (pointSet.getMinDate()>calendarParam.getCycle()) {
						return false;
					}else {
					calendarPointSet.setMinDate(pointSet.getMinDate());
					calendarPointSet.setPoint(pointSet.getPoint());
					}
				}
				calendarPointSet.setCreateById(accountId);
				calendarPointSet.setCreateByName(accountName);
				calendarPointSet.setCreateTime(new Date());
				boolean b = calendarPointSetService.insert(calendarPointSet);
			}
			return true;
		}else {
			//修改日历打卡规则
			calendar.setSetType(calendarParam.getSetType());
			calendar.setFirstPoint(calendarParam.getFirstPoint());
			calendar.setCycle(calendarParam.getCycle());
			calendar.setState(calendarParam.getState());
			calendar.setUpdateById(accountId);
			calendar.setUpdateByName(accountName);
			calendar.setUpdateTime(new Date());
			List<CalendarPointSet> list = new ArrayList<CalendarPointSet>();
			for(CalendarPointSetParam pointSet:calendarParam.getCalendarPointSets()) {
				CalendarPointSet calendarPointSet = new CalendarPointSet();
				calendarPointSet.setCalendarId(calendar.getId());
				calendarPointSet.setId(idGenerator.generate());
				calendarPointSet.setSetType(calendarParam.getSetType());
				if (calendarParam.getSetType()==1) {
					calendarPointSet.setPoint(pointSet.getPoint());
				}else {
					if (calendarParam.getCycle()<=pointSet.getMinDate()) {
						return false;
					}else {
					calendarPointSet.setMinDate(pointSet.getMinDate());
					calendarPointSet.setPoint(pointSet.getPoint());
					}
				}
				calendarPointSet.setCreateById(accountId);
				calendarPointSet.setCreateByName(accountName);
				calendarPointSet.setCreateTime(new Date());
				list.add(calendarPointSet);
			}
			CalendarPointSet calendarPointSet = new CalendarPointSet();
			calendarPointSet.setCalendarId(calendar.getId());
			EntityWrapper<CalendarPointSet> entityWrapper = new EntityWrapper<CalendarPointSet>(calendarPointSet);
			//删除之前的积分规则
			calendarPointSetService.delete(entityWrapper);
			//新的积分规则入库
			calendarService.updateById(calendar);
			if (!CollectionUtils.isEmpty(list)) {
				boolean b = calendarPointSetService.insertBatch(list);
			}
			return true;
		}
	}
}

