package com.yizhi.calendar.application.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.yizhi.calendar.application.domain.Calendar;
import com.yizhi.calendar.application.domain.CalendarPointSet;
import com.yizhi.calendar.application.domain.CalendarRecord;
import com.yizhi.calendar.application.service.ICalendarPointSetService;
import com.yizhi.calendar.application.service.ICalendarService;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.util.application.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.calendar.application.service.ICalendarRecordService;
import com.yizhi.calendar.application.vo.CalendarSignViewVO;
import com.yizhi.calendar.application.vo.HeadVO;

/**
 * <p>
 * 学员日历打卡
 * </p>
 *
 * @author fulan123
 * @since 2019-10-10
 */
@RestController
@RequestMapping("/api/calendar")
public class StuentCalendarController {
	
	private static final Logger LOG = LoggerFactory.getLogger(StuentCalendarController.class);
	
	@Autowired
	private ICalendarService calendarService;
	
	@Autowired
	private ICalendarPointSetService calendarPointSetService;
	
	@Autowired
	private ICalendarRecordService calendarRecordService;
	
	@Autowired
	private ReportClient reportClient;
	/**
	 * 打卡签到入口
	 * @param accountId
	 * @param siteId
	 * @return
	 */
	@GetMapping("/sign")
	public Map<String, Object> sign(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId) {
		Map<String, Object> resultMap = new HashMap<String, Object>();
		Calendar set = new Calendar();
		set.setSiteId(siteId);
		EntityWrapper<Calendar> wrapper = new EntityWrapper<Calendar>(set);
		Calendar calendar = calendarService.selectOne(wrapper);
		if (null!=calendar) {
			CalendarPointSet pointSet = new CalendarPointSet();
			pointSet.setCalendarId(calendar.getId());
			pointSet.setSetType(calendar.getSetType());
			EntityWrapper<CalendarPointSet> entityWrapper = new EntityWrapper<CalendarPointSet>(pointSet);
			List<CalendarPointSet> list = calendarPointSetService.selectList(entityWrapper);
			if (CollectionUtils.isNotEmpty(list)) {
				Map<String, Object> map = calendarRecordService.sign(accountId,siteId,calendar,list);
				LOG.info(map.get("result").toString());
				return map;
			}else {
				resultMap.put("result", "签到失败");
				LOG.info(siteId+"---该站点暂未开启日历打卡签到功能");
				return resultMap;
			}
		}else {
			resultMap.put("result", "签到失败");
			LOG.info(siteId+"---该站点暂未设置打卡积分发放规则");
			return resultMap;
		}
	}

	/**
	 * 用户每日打卡详情
	 * @param accountId
	 * @param siteId
	 * @return
	 */
	@GetMapping("/isSign")
	public CalendarSignViewVO isSign(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId) {
		CalendarSignViewVO vo = new CalendarSignViewVO();
		Calendar set = new Calendar();
		set.setSiteId(siteId);
		EntityWrapper<Calendar> wrapper = new EntityWrapper<Calendar>(set);
		Calendar calendar = calendarService.selectOne(wrapper);
		if (null!=calendar) {
			CalendarPointSet pointSet = new CalendarPointSet();
			pointSet.setCalendarId(calendar.getId());
			pointSet.setSetType(calendar.getSetType());
			EntityWrapper<CalendarPointSet> entityWrapper = new EntityWrapper<CalendarPointSet>(pointSet);
			List<CalendarPointSet> list = calendarPointSetService.selectList(entityWrapper);
			if (CollectionUtils.isNotEmpty(list)) {
				vo= calendarRecordService.isSign(accountId,siteId,calendar,list);
				LOG.info("用户："+accountId+"每日签到详情："+vo);
				return vo;
			}else {
				LOG.info(siteId+"---该站点暂未设置打卡积分发放规则");
				return vo;
			}
		}else {
			LOG.info(siteId+"---该站点暂未开启日历打卡签到功能");
			return vo;
		}
	}
	
	@GetMapping("/headData")
	public HeadVO getHeadData(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId) {
		return calendarRecordService.getHeadData(accountId, siteId);
	}
	
	@GetMapping("/signDetail")
	public Page<CalendarRecord> getSignDetail(@RequestParam("accountId") Long accountId, @RequestParam("siteId") Long siteId, @RequestParam("type") Integer type, @RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize){
		Page<CalendarRecord> page = calendarRecordService.getSignDetailByAccountId(accountId, siteId, type, pageNo, pageSize);
		return page;	
	}
	
	@GetMapping("/signByMonth")
	public List<CalendarRecord> getSignByMonth(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId,@RequestParam("month") String month){
		 List<CalendarRecord> list = calendarRecordService.getSignByMonth(month, accountId, siteId);
		return list;
	}
	
	@GetMapping("/signNum")
	public Map<Long, Long> signNum(@RequestParam("siteIds") List<Long> siteIds,
			@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate){
		return calendarRecordService.signNum(siteIds, startDate, endDate);
	}
	
	@GetMapping("/getSignState")
	public Integer getSignState(@RequestParam("accountId") Long accountId,@RequestParam("siteId") Long siteId) {
		Date date = new Date();
		String dayString = DateUtil.toDay(date);
		CalendarRecord record = new CalendarRecord();
		record.setSignDate(dayString);
		record.setAccountId(accountId);
		record.setSiteId(siteId);
		EntityWrapper<CalendarRecord> wrapper = new EntityWrapper<CalendarRecord>(record);
		CalendarRecord one = calendarRecordService.selectOne(wrapper);
		if (one!=null) {
			return 1;
		}else {
			return 0;
		}
	}
}
