package com.yizhi.calendar.application.mapper;

import com.yizhi.calendar.application.domain.CalendarRecord;
import com.yizhi.calendar.application.vo.CalendarChartVO;
import com.yizhi.calendar.application.vo.CalendarDetailStatisticVO;
import com.yizhi.calendar.application.vo.CalendarStatisticVO;
import com.yizhi.calendar.application.vo.HeadVO;
import com.yizhi.calendar.application.vo.SignNumVO;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.baomidou.mybatisplus.mapper.BaseMapper;

/**
 * <p>
 * 日历打卡记录表 Mapper 接口
 * </p>
 *
 * @author fulan123
 * @since 2019-10-10
 */
public interface CalendarRecordMapper extends BaseMapper<CalendarRecord> {

	/**
	 * 获取用户最近一次签到记录
	 * 
	 * @param accountId
	 * @return
	 */
	List<CalendarRecord> getLastSignByaccountId(@Param("accountId") Long accountId, @Param("siteId") Long siteId);

	/**
	 * 获取用户最后一次首次签到记录
	 * 
	 * @param accountId
	 * @return
	 */
	List<CalendarRecord> getLastFirstSignByaccountId(@Param("accountId") Long accountId, @Param("siteId") Long siteId);

	/**
	 * 获取最大连续签到天数
	 * 
	 * @param calendarId
	 * @param minDate
	 * @return
	 */
	Integer getMaxDate(@Param("calendarId") Long calendarId, @Param("minDate") Integer minDate);

	/**
	 * 获取累计天数和总积分
	 * 
	 * @param accountId
	 * @param siteId
	 * @return
	 */
	HeadVO getSumDayAndPoint(@Param("accountId") Long accountId, @Param("siteId") Long siteId);

	/**
	 * 学员端签到明细
	 * 
	 * @param accountId
	 * @param siteId
	 * @param type
	 * @return
	 */
	List<CalendarRecord> getSignDetailByAccountId(@Param("accountId") Long accountId, @Param("siteId") Long siteId,
			@Param("type") Integer type);

	/**
	 * 日历详情
	 * 
	 * @param date
	 * @return
	 */
	List<CalendarRecord> getSignByMonth(@Param("month") String month, @Param("accountId") Long accountId,
			@Param("siteId") Long siteId);

	/**
	 * 折线图
	 * 
	 * @param startDate
	 * @param endDate
	 * @param companyId
	 * @param siteId
	 * @return
	 */
	List<CalendarChartVO> signChart(@Param("startDate") String startDate, @Param("endDate") String endDate,
			@Param("companyId") Long companyId, @Param("siteId") Long siteId);

	List<CalendarStatisticVO> list(@Param("startDate") String startDate, @Param("endDate") String endDate,
                                   @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                   @Param("accountIds") List<Long> accountIds);

	List<CalendarRecord> accountList(@Param("startDate") String startDate, @Param("endDate") String endDate,
			@Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("accountId") Long accountId);

	List<CalendarDetailStatisticVO> detailList(@Param("startDate") String startDate, @Param("endDate") String endDate,
			@Param("companyId") Long companyId, @Param("siteId") Long siteId,
			@Param("accountIds") List<Long> accountIds);

	List<SignNumVO> signNum(@Param("siteIds") List<Long> siteIds,
                            @Param("startDate") String startDate, @Param("endDate") String endDate);
}
